*------------------------------
	subroutine chi2map( option )
* make chi2_12( n_1, n_2 ) = chi^2 map vs planet position
* assumes par_1( n_1 ) and par_2( n_2 ) have been set.
* 1998 Aug Keith Horne @ St-Andrews
* 2002 Apr KDH @ St-And - optimize 4 params
* 2002 Apr KDH @ St-And - cpu
* 2005 May KDH @ St-And - checkcxy, stowcxy
* 2007 Apr KDH @ St-And - fit 5 lens parameters (with blending)
* 2007 Aug KDH @ LaSilla - adapt from cxymap
* 2009 Jul KDH @ StA - t0 -> hjd0, premag(t) -> premag(hjd)
* 2011 Mar KDH @ StA - stow maptweak, mapfit, mapdof, chi2ref, cpumap
	include 'plens.inc'
	character*(*) option
	logical useoldmap
	logical fast/.true./
	call upcase( option, opt )
	fast = .false.
	fast = .true.
	if( opt .eq. 'FQ' ) fast = .false.
* report
	write(*,*) 'chi2map( ' // opt(:len1(opt)) // ' ) ***********'
	write(*,*) 'fast =', fast
	call showgrid( opt )

* use one we made earlier
      if( useoldmap( opt ) ) then
	write(*,*) 'Using previous Chi^2( ', opt(1:2), ' )'
	call fetchgrid( opt )
	return
      else
	write(*,*) 'Must recompute Chi^2( ', opt(1:2), ' )'
      end if

* stow planet and npfit
	call saveplanet
	npfsave = npfit
* ditch planet
	call noplanet
* noplanet fit of 5 lens parameters
	npfit = 5
	call fit1( nits, npfit, bof ) 
* stow lens parameters
	call savepar( npfit )
* chi^2 with no planet
	call chi2calc( chi2min, ndf )
* degrees of freedom
	ndfit = ndata
	ndof = max( 0, ndfit - npfit )
* report reduced chi^2
	chi2red = chi2min / max(1,ndof)
	write(*,*) 'WITHOUT planet, parameters:', npfit, ' bof:', bof
	write(*,*) '                   chi2min:', chi2min,
     &	' / (', ndfit, ' -', npfit, ' ) =', chi2red

* stow
	mapfit = npfit
	mapdof = ndof
	chi2ref = chi2min
	chi2fit = chi2min
* set cpu
	call cpu( dcpu )
	cpudone = 0.
	cpufast = 0.

* number of parameters to tweak
	npfit = ntweak
	call restoreplanet
	repl = sqrt( qpl )

* fast preliminary assessment
      if( fast ) then
	write(*,*) ' FAST reconnaisance using IMCLOSE.'

* image positions ( used by imclose )
	write(*,*) 'Image positions for u0', y0, ' A0', a0, ' data', ndata
	call xcalc
	call lcalc( ndata, x_t, y0 )
	call mnmx( ndata, xp_t, xmn, xmx )
	call mnmx( ndata, yp_t, ymn, ymx )
	call mnmx( ndata, ap_t, amn, amx )
	write(*,*) 'Major: x', xmn, xmx, ' y', ymn, ymx, ' A', amn, amx
	call mnmx( ndata, xm_t, xmn, xmx )
	call mnmx( ndata, ym_t, ymn, ymx )
	call mnmx( ndata, am_t, amn, amx )
	write(*,*) 'Minor: x', xmn, xmx, ' y', ymn, ymx, ' A', amn, amx

* mag error bars ( used by imclose )
	nloop = 2
      do loop = 1, nloop
      if( loop .eq. nloop ) then
	write(*,*) 'call premag, erbmag'
      do i=1,ndata
	pmag = premag( hjd_t( i ) )
	emag = erbmag( i, pmag )
	erbmag_t( i ) = emag
      end do
      end if
	call mnmx( ndata, erbmag_t, smn, smx )
	write(*,*) 'Magnitude errors:', smn, smx
      end do

* initialize chi2 map
	n_12 = n_1 * n_2
	call peqc( n_12, chi2_12, chi2fit )
	write(*,*) 'Chi^2( ', opt(1:2), ' )', n_12, ' pixels set to', chi2fit

* loop over pixels
	nsum = 0
	i = 0
      do i2 = 1, n_2
	a2 = par_2( i2 )
      do i1 = 1, n_1
	a1 = par_1( i1 )
	i = i + 1
* set the planet
	call setplanet( opt, a1, a2 )
* is planet close to an image ?
      if( imclose( upl, apl ) .ne. 0 ) then
	nsum = nsum + 1
* mark for evaluation
	chi2_12( i ) = -1.
      end if
* next i1
      end do
* report
	nreport = 10
      if( i2 .eq. 1 .or. i2 .eq. n_2
     &	.or. mod( i2, max( 1, n_2 / nreport ) ) .eq. 0 ) then
	p = nsum * 100. / ( n_1 * i2 )
	write(*,*) i2, n_2, ' At ', opt(2:2), ' =', par_2( i2 )
     &		, ' need', nsum, ' of', n_1 * i2
     &		, ' =', nint( p ), ' %'
      end if
* next i2
      end do
* cpu report
	call cpu( dcpu )
	cpufast = cpufast + dcpu
	write(*,*) ' FAST reconnaisance using IMCLOSE. CPU:', cpufast
      else
	nsum = n_12
      end if
* report
	p = nsum * 100. / n_12
	write(*,*) 'Need Chi^2( ', opt(1:2), ' ) at '
     &		, nsum, ' of', n_12, ' =', nint( p ), ' %'
	write(*,*) '   tweaking ', ntweak, ' parameters.'

* set planet and evaluate change in chi2
	call restoreplanet
	repl = sqrt( qpl )
c	write(*,*) 'Planet at', xpl, ypl, ' q=', qpl
* loop over pixels
	i = 0
	itop = 1
	ibot = 1
      do i2 = 1, n_2
	a2 = par_2( i2 )
      do i1 = 1, n_1
	a1 = par_1( i1 )
	i = i + 1
      if( .not. fast .or. chi2_12( i ) .le. 0. ) then
* set the planet
	call setplanet( opt, a1, a2 )
* evaluate badness of fit
	bof = chi2fit
* restore lens parameters
	call restorepar( npfit )
* evaluate fit
	call chi2calc( bof, ndf )
* debug report
c      if( i1 .eq. 1 .or. i1 .eq. n_1 ) then
c	write(*,*) ' Planet at', xpl, ypl, ' bof', bof, ' ndf', ndf
c      end if
* fit lens parameters with planet in place - only if necessary
      if( npfit .gt. 0 ) then
	test = abs( bof - chi2fit ) / delchi2
	bigtest = 0.02
      if( test .gt. bigtest ) then
c	nrefit = nrefit + 1
	call fit1( nits, npfit, bof ) 
	call chi2calc( bof, ndf )
      end if
      end if
* stow
	chi2_12( i ) = bof
	if( bof .lt. chi2_12( itop ) ) itop = i
	if( bof .gt. chi2_12( ibot ) ) ibot = i
      end if
* next i1
      end do

* check cpu
	call cpu( dcpu )
	cpudone = cpudone + dcpu
	cputogo = ( n_2 - i2 ) * cpudone / i2
	cputogo2 = ( n_2 - i2 ) * dcpu
* report
	nreport = 30
      if( i2 .eq. n_2 .or. i2 .eq. 1
     &	.or. mod( i2, max( 1, n_2 / nreport ) ) .eq. 0 ) then
	call mnmx( n_1, chi2_12( i - n_1 + 1 ), cmn, cmx )
	cpuguess = cpudone + ( cputogo + cputogo2 ) / 2.
	p = 100. * cpudone
	if( cpuguess .gt. 0. ) p = p / cpuguess
	write(*,*) i2, n_2, npfit, ' ', opt(2:2), a2, ' chi2', cmn, cmx
     &	, ' cpu:', nint( cpudone ), nint( cputogo ), nint( cputogo2 )
     &	, nint( p ), ' %'
      end if
* next i2
      end do
	cputotal = cpufast + cpudone
	write(*,*) 'Total CPU:', cpufast, ' +', cpudone, ' =', cputotal

* report best chi2 with planet
	call mnmx( n_12, chi2_12, chi2min, chi2max )
	nd = ndata
	np = max( npfsave, npfit ) + 2
	ndof = max( 0, nd - np )
	chi2red = chi2min / max( 1, ndof )
	write(*,*) 'WITH planet Chi2min =', chi2min
     &		, ' / (',  nd, ' -', np, ' ) =', chi2red

* report worst
	i2 = 1 + ( ibot - 1 ) / n_1
	i1 = ibot - n_1 * ( i2 - 1 )
	a1 = par_1( i1 )
	a2 = par_2( i2 )
	dchi2fit = chi2fit - chi2_12( ibot )
	write(*,*) 'Worst DelChi2( ' 
     &		, opt(1:1), ' =', a1, ', ', opt(2:2), ' =', a2
     &		, ' ) =', dchi2fit
* report best
	i2 = 1 + ( itop - 1 ) / n_1
	i1 = itop - n_1 * ( i2 - 1 )
	a1 = par_1( i1 )
	a2 = par_2( i2 )
	chi2best = chi2_12( itop )
	dchi2fit = chi2fit - chi2best
	write(*,*) ' Best DelChi2( ' 
     &		, opt(1:1), ' =', a1, ', ', opt(2:2), ' =', a2
     &		, ' ) =', dchi2fit

* set planet to best
	call setplanet( opt, a1, a2 )

* stow (SHOULD MAKE GENERIC SLOTS TO STOW THESE FITS)

	qmap = qpl
	maptweak = ntweak
	cpumap = cputotal

	call mnmx( n_12, chi2_12, cmn, cmx )
	write(*,*) 'Chi2( ', opt(1:2), ' )', cmn, cmx
	call stowgrid( opt )
	call mnmx( n_12, chi2_12, cmn, cmx )
	write(*,*) 'Chi2( ', opt(1:2), ' )', cmn, cmx

	smag0fit = smag0
	b0fit = b0
c	t0fit = t0
	hjd0fit = hjd0
	tefit = te
	a0fit = a0
	y0fit = y0
	xplfit = xpl
	yplfit = ypl
	qplfit = qpl

* restore previous planet
	call restoreplanet
	npfit = npfsave
* summarize chi map
	call cmapsummary( 20 )
	return
	end
