*------------------------------
	subroutine fit0( nits, ftol, nloop, nfit, bof )
* amoeba fit
* Input:
*	nits	i4 max number of iterations
*	ftol	r4 fit tolerance
*	nloop	i4 number of loops
*	nfit	i4 number of parameters to fit
*	abof	external badness of fit function
* Output:
*	nits	i4 actual number of iterations
*	bof	r4 badness of fit
* 1998 Sep Keith Horne @ St-Andrews
* 2003 Nov KDH @ St-And - seecor, ftol, nloop
* 2006 Feb KDH @ St-And - skip fit if ndata < 1
* 2007 Apr KDH @ St-And - skip linear params 1 and 5
* 2009 Jul KDH @ St-And - muteamoebaplot
	include 'plens.inc'
* simplex vertices
        parameter (maxpar=200)
        parameter (maxvert = maxpar+1 )
	real*4 par( maxpar ), dpar( maxpar )
        real*4 pvert( maxvert * maxpar )
        real*4 bvert( maxvert )
	logical verbose/.false./
	logical silence/.false./
	external abof
	common/ nonlinpar / nonlinpar
	common/ linearpar / linearpar
        ivp( iv, ip ) = iv + ( ip - 1 ) * nvert

* keep quiet ?
	silence = mutefit0 .ne. 0

	npfit = nfit
* max number of iterations
	mxiter = nits

      if( .not. silence ) then
	write(*,*)
	write(*,*) 'fit0(', nits, ftol, nloop, nfit, ' )'
     &	, ' data:', ndata
     &	, ' ** ', datfile(:max(1,len1(datfile)) ), ' ** '
      end if
      if( ndata .le. 0 ) then
	write(*,*) '** WARNING in fit0: NO DATA TO FIT.'
	return
      end if
* insufficient data for requested fit ?
      if( nfit .gt. ndata ) then
	write(*,*) '** WARNING in fit0: nfit', nfit, '> ndata', ndata
	nits = 0
      end if

* report parameters
c	write(*,*) 'before safe m0', smag0, ' b0', b0, ' sig0', sigmag0
	call safe( par, dpar )
c	write(*,*) 'after safe m0', smag0, ' b0', b0, ' sig0', sigmag0

	npar = nonlinpar
	bof = abof( par )
c	write(*,*) 'after abof m0', smag0, ' b0', b0, ' sig0', sigmag0
      if( .not. silence ) then
	write(*,*) 'Amoeba initial parameters:', npar, ' BoF =', bof
	write(*,*) ( par( i ), i = 1, npar )
      end if
* loops
	iterations = 0
      do loop = 1, nloop

* form simplex
	nvert = npar + 1
      do ip=1,npar
	delta = dpar(ip)
      do iv=1,nvert
	pvert( ivp(iv,ip) ) = par( ip ) - delta / nvert
      end do
	pvert( ivp(ip,ip) ) = pvert( ivp(ip,ip) ) + delta
      end do

* badness at each vertex
      do iv=1,nvert
      do ip=1,npar
	par(ip) = pvert( ivp( iv, ip ) )
      end do
	bvert(iv) = abof( par )
      end do

* report
      if( verbose ) then
	call mnmx( nvert, bvert, bmn, bmx )
	write(*,*) 'vertices:', nvert, ' span:', bmn, bmx, bmx-bmn
      do iv = 1,nvert
	write(*,*) bvert(iv), (pvert( ivp(iv,ip) ) ,ip=1,npar)
      end do
	write(*,*) 'Amoeba iterating....'
      end if

* adjust simplex
	niter = mxiter
c	write(*,*) 'before amoeba m0', smag0, ' b0', b0, ' sig0', sigmag0
        call amoeba( pvert, bvert, nvert,
     &		npar, npar, ftol, abof, niter )
	iterations = iterations + niter
c	write(*,*) 'after amoeba m0', smag0, ' b0', b0, ' sig0', sigmag0

* diagnostic plot
	if( muteamoebaplot .eq. 0 ) call amoebaplot( ' ' )

* report final simplex
      if( verbose .or. .not. silence ) then
	write(*,*) 'Amoeba finished .............. iterations:', iterations
	call mnmx( nvert, bvert, bmn, bmx )
	write(*,*) 'vertices:', nvert, ' span:', bmn, bmx, bmx-bmn
      do i = 1, nvert
	write(*,*) bvert(i), ( pvert( ivp(i,ip) ) ,ip=1,npar)
      end do
      end if

* find best vertex
* try to avert NAN
	call mnmx( nvert, bvert, bmn, bmx )
	best = bmx
      do i=1,nvert
	b = bvert(i)
      if( b .le. best ) then
	ivbest = i
	best = b
      end if
	if( verbose ) write(*,*) i, ivbest, b, bvert(i), best
      end do
* stow best-fit parameters
	bof = best
      do i=1,npar
	par( i ) = pvert( ivp( ivbest, i ) )
      end do
* reconstruct parameters
c	write(*,*) 'before desafe m0', smag0, ' b0', b0, ' sig0', sigmag0
	call desafe( par )
c	write(*,*) 'after desafe m0', smag0, ' b0', b0, ' sig0', sigmag0
* scale factors

C SHOULD WE RE-FIT THE FLUXES AT THIS STAGE ?
	call fitfsfb
c	write(*,*) 'after fitfsfb m0', smag0, ' A0', a0, ' b0', b0, ' sig0', sigmag0
* seeing corrections
	call seecor
c	write(*,*) 'after seecor m0', smag0, ' A0', a0, ' b0', b0, ' sig0', sigmag0
* report
      if( .not. silence ) then
	write(*,*) 'Amoeba final parameters:', npar, ' BoF =', bof
	write(*,*) ( par( i ), i = 1, npar )
      end if
* report
      if( nloop .gt. 1 ) then
      if( loop .eq. 1 ) then
	write(*,*) 'Amoeba LOOP', loop
     &		, ' its', niter, iterations, ' BoF', bof
      else
	write(*,*) 'Amoeba LOOP', loop
     &		, ' its', niter, iterations, ' BoF', bof
     &		, ' change', bof - bofold
      end if
	bofold = bof
      end if

* next loop
      end do

* total 
	nits = iterations
	return
	end
