*------------------------------
	subroutine gridfit( option )
* chi^2 grid fit
* 2007 Aug Keith Horne @ La Silla
	include 'plens.inc'
	character*(*) option
	call upcase( option, opt )
	write(*,*) 'gridfit( ' // opt(:len1(opt)) // ' ) ***********'
* lightcurve
	call xcalc
	call lcalc( ndata, x_t, y0 )
* grid
	call gridbox( opt )
* chi^2 map
	call chi2map( opt )

* compute relative probabilities
	n_12 = n_1 * n_2
	call mnmx( n_12, chi2_12, cmn, cmx )
	i = 0
      do i2 = 1, n_2
      do i1 = 1, n_1
	i = i + 1
	post_12( i ) = exp( -0.5 * ( chi2_12( i )  - cmn ) )
      end do
      end do
* project onto axis 2
	i = 1
      do i2 = 1, n_2
	sum = 0.d0
	cmn = chi2_12( i )
      do i1 = 1, n_1
	sum = sum + post_12( i )
	cmn = min( cmn, chi2_12( i ) )
	i = i + 1
      end do
	chi2_2( i2 ) = cmn
	post_2( i2 ) = sum
      end do
* project onto axis 1
      do i1 = 1, n_1
	i = i1
	sum = 0.d0
	cmn = chi2_12( i1 )
      do i2 = 1, n_2
	sum = sum + post_12( i )
	cmn = min( cmn, chi2_12( i ) )
	i = i + n_1
      end do
	chi2_1( i1 ) = cmn
	post_1( i1 ) = sum
      end do
* scale projected probabilities to peak 1
	call mnmx( n_1, post_1, pmn, pmx )
	if( pmx .gt. 0. ) call peqpmc( n_1, post_1, post_1, 1./pmx )
	call mnmx( n_2, post_2, pmn, pmx )
	if( pmx .gt. 0. ) call peqpmc( n_2, post_2, post_2, 1./pmx )
* report
	x1 = par_1( 1 )
	x2 = par_1( n_1 )
	y1 = par_2( 1 )
	y2 = par_2( n_2 )
	dx = ( x2 - x1 ) / max( 1, n_1 - 1 )
	dy = ( y2 - y1 ) / max( 1, n_2 - 1 )
	write(*,*) opt(1:1), ':', n_1, x1, x2, dx
	write(*,*) opt(2:2), ':', n_2, y1, y2, dy
* window
	if( opt( 1:1 ) .eq. 'T' ) call timeaxis( xmn, xmx, xlabel )
	call mnmx( n_1, par_1, xmn, xmx )
	call mnmx( n_2, par_2, ymn, ymx )
	dx = ( xmx - xmn ) / max( 1, n_1 - 1 )
	dy = ( ymx - ymn ) / max( 1, n_2 - 1 )
	add = dx / 2.
	xmn = xmn - add
	xmx = xmx + add
	add = dy / 2.
	ymn = ymn - add
	ymx = ymx + add
	write(*,*) 'x', xmn, xmx, ' y', ymn, ymx

	call pgsci( kolor_window )

* log u and angle
      if( opt .eq. 'UA' ) then
	xx = abs( xmx - xmn ) * alog( 10. )
* trim a sliver from the azimuth degree range
	tiny = 0.01
	ymn = nint( ymn ) + tiny
	ymx = nint( ymx ) - tiny
	yy = deg2rad( abs( ymx - ymn ) )
	call pgenv( 0., xx, 0., yy, 1, -2 )	
	call pgswin( xmn, xmx, ymn, ymx-0.01 )
	call pgbox( 'bcnstl', 0., 0,  'bcnst', 90., 3 )
	cpugrid = cpuua
* x and y
      else if( opt .eq. 'XY' ) then
	call pgenv( xmn, xmx, ymn, ymx, 1, 0 )
	cpugrid = cpuxy
* f and q
      else if( opt .eq. 'FQ' ) then
	call pgenv( xmn, xmx, ymn, ymx, 0, 20 )
	cpugrid = cpufq
* t and f
      else
	call pgenv( xmn, xmx, ymn, ymx, 0, 0 )
	cpugrid = cputf
      end if

* labels
      if( opt .eq. 'XY' ) then
	xlabel = 'x/R\dE\u'
	ylabel = 'y/R\dE\u'
      else if( opt .eq. 'TF' ) then
	ylabel = 'f = ( u\dp\u / u\di\u )'
      else if( opt .eq. 'FQ' ) then
	xlabel = 'f = ( u\dp\u / u\di\u )'
	ylabel = 'q = m / M'
      else if( opt .eq. 'UA' ) then
	xlabel = 'r/r\dE\u'
	ylabel = '\ga (degrees)'
      end if
	call pgsci( kolor_label )
	call pglabel( xlabel, ylabel, ' ' )
* title
	title = ' '
	call title_event( title )
c	title = datfile(:len1(datfile))
	call title_q( title )
	call title_delchi2( title )
	call title_best( title )
	call pgsci( kolor_label )
	call pglabel( ' ', ' ', title )

* transform
	tr( 6 ) = dy
	tr( 5 ) = 0.
	tr( 4 ) = y1 - dy
	tr( 3 ) = 0.
	tr( 2 ) = dx
	tr( 1 ) = x1 - dx
* greyscale
	fg = chi2fit + delchi2
	bg = chi2fit - delchi2
	bg = max( 0., bg )
	call mnmx( n_12, chi2_12, cmn, cmx )
	write(*,*) 'Chi2( ', opt(1:1), ', ', opt(2:2), ' )', cmn, cmx
     &		, ' chi2fit', chi2fit

* relative probability map
      if( opt .eq. 'FQ' ) then
	bg = 1.
	fg = 0.
	call pggray( post_12, n_1,n_2, 1,n_1, 1,n_2, bg, fg, tr )
* chi^2 map
      else
	write(*,*) n_1, n_2, ' greyscale bg', bg, ' fg', fg
	call pgsci(1)
	call pggray( chi2_12, n_1,n_2, 1,n_1, 1,n_2, bg, fg, tr )
      end if
* calibration wedge
	call pgsci( kolor_window )
	call pgwedg('r', 0.2, 3., bg, fg, ' ' )

* legend
	label = 'fit'
	call append_i( label, mapfit )
	call append( label, 'tweak' )
	call append_i( label, maptweak )

	call pgsci( kolor_label )
	call pgsch( 0.7 * csize )
	call pgmtxt( 'T', -1.5, 0.98, 1.0, label )

	label = '\gx\u2\d'
	call append_f( label, chi2ref, '(f10.2)' )
	call append( label, '/' )
	call append_i( label, mapdof )
	call pgmtxt( 'T', -3.0, 0.98, 1.0, label )
c	call pgmtxt( 'T', 0.5, 0., 0., label )
	call pgsch( csize )

* redraw box

	call pgsci( kolor_window )
      if( opt .eq. 'UA' ) then
	call pgbox( 'bcstl', 0., 0, 'bcst', 90., 3 )
      else if( opt .eq. 'FQ' ) then
	call pgbox( 'bcst', 0., 0, 'bcstl', 0., 0 )
* chi^2 contours
      do k=1,3
	c = cmn + k * k
	call pgsci( i )
	call pgcont( chi2_12, n_1,n_2, 1,n_1, 1,n_2, c, -1, tr )
      end do
      else

	call pgbox( 'bcst', 0., 0, 'bcst', 0., 0 )
      end if
* horizontals
* tf
      if( opt .eq. 'TF' ) then
	call pgsls( 4 )
      do k=-1,1
	y = k
	call pgmove( xmn, y )
	call pgdraw( xmx, y )
      end do
	call pgsls( 1 )
* ua
      else if( opt .eq. 'UA' ) then
      do i = -90, 270, 180
	y = i
	call pgmove( xmn, y )
	call pgdraw( xmx, y )
      end do
	call pgsls( 4 )
      do i = -180, 180, 180
      do k=-1,1
	call xy2deg( float(k), y0, deg )
	y = i + deg
	call pgmove( xmn, y )
	call pgdraw( xmx, y )
      end do
      end do
      end if
* verticals
* xy
      if( opt .eq. 'XY' ) then
* fq
      else if( opt .eq. 'FQ' ) then
	call pgsls( 4 )
      do k=-1,1
	x = k
	call pgmove( x, ymn )
	call pgdraw( x, ymx )
      end do
* tf
      else if( opt .eq. 'TF' ) then
	call pgsls( 4 )
	call marktime( ymn, ymx )
	call pgsls( kolor_now )
	call marknow( ymn, ymx )
* ua
      else if( opt .eq. 'UA' ) then
	call pgsls( 1 )
	call pgmove( 0., ymn )
	call pgdraw( 0., ymx )
      end if
	call pgsls( 1 )

* legend
c	call pgsci( kolor_label )
c	call legend

* cpu
	cpu = -1.
      if( opt .eq. 'UA' ) then
	cpu = cpuua
      else if( opt .eq. 'XY' ) then
	cpu = cpuxy
      else if( opt .eq. 'FQ' ) then
	cpu = cpufq
      else if( opt .eq. 'TF' ) then
	cpu = cputf
      end if
	call pgsci( 4 )
	call pgcpu( cpu )

* einstein ring
	call pgsci( kolor_model )
	call ering( opt )

* image trajectories
	call pgsci( kolor_model )
	call pgsls( 4 )
      if( opt .eq. 'UA' ) then
	call impathal( ymn, ymx, 360., 4, 4 )
      else if( opt .eq. 'XY' ) then
	call impaths( 4, 4 )
      else if( opt .eq. 'TF' ) then
	call pgsls( 4 )
      do k=-1,1
	y = k
	call pgmove( xmn, y )
	call pgdraw( xmx, y )
      end do
      end if
	call pgsls( 1 )

* image positions
c	call pgsci( kolor_model )
c	call marksiteal( xmn, xmx, 360., 1 )

* planet
      if( dchi2fit .gt. delchi2 ) then
	call saveplanet
	qpl = qplfit
	call setplxy( xplfit, yplfit )
	title = ' '
	call title_plpos( title )
	call pgsci( kolor_label )
	call pgmtxt( 't', 0.5, 0.5, 0.5, title )
	call pgsls( 2 )
	call pgsci( kolor_window )
	iadd1 = 0
	iadd2 = 0
	iadd = 1
      if( opt .eq. 'XY' ) then
	x = xpl
	y = ypl
      else if( opt .eq. 'TF' ) then
	x = tpl
	y = fpl
      else if( opt .eq. 'FQ' ) then
	x = fpl
	y = alog10( qpl )
      else if( opt .eq. 'UA' ) then
	x = alog10( upl )
	y = rad2deg( apl )
	iadd1 = -360
	iadd2 = 360
	iadd = 360
      end if


	call pgsci( 3 )
      do i = iadd1, iadd2, iadd
	yy = y + i
	call pgmove( xmn, yy )
	call pgdraw( xmx, yy )
      end do
	call pgmove( x, ymn )
	call pgdraw( x, ymx )
	call pgsls( 1 )
	call restoreplanet
	call planet( opt )
      end if

* plot projected probabilities
      if( dchi2fit .gt. 25. ) then
	fac = 0.1
      do i = 1, n_1
	plot1(i) = ymn + fac * post_1(i) * (ymx - ymn )
      end do
      do i = 1, n_2
	plot2(i) = xmn + fac * post_2(i) * (xmx - xmn )
      end do
	call pgsci( 3 )
	call pgbin( n_1, par_1, plot1, .true. )
	call pgbin( n_2, plot2, par_2, .true. )
      end if

	return
	end
