*------------------------------
	subroutine guesslens
* estimate lens parameters from time and magnitude data
* 1998 Aug Keith Horne @ St-Andrews
* 2002 Apr KDH @ St.And - improve estimate of te
* 2002 Nov KDH @ St.And - b0
* 2002 Nov KDH @ St.And - quantiles, more robust guesses
* 2003 Apr KDH @ St.And - guess te more robustly
* 2007 Mar KDH @ Goettingen - grid search on t0, te
* 2007 Apr KDH @ St.And - plots to test
* 2008 Jul KDH @ StA - adjust te to match mag between peak and baseline
* 2010 Mar KDH @ StA stow siglogf_l and avglogf_l
* 2010 Apr KDH @ StA revise diagnostic plot
* 2010 Apr KDH @ StA prior(logte)
* 2010 May KDH @ StA baseline median mag on diagnostic plot
* 2010 May KDH @ StA use median mag if fainter, adjust a0,te accordingly
	include 'plens.inc'
	real*8 sum, sum0
	real*8 sump, sumpt0, sumpte
	logical verbose/.false./
	logical plots/.false./
	logical pgagain
	write(*,*)
	write(*,*) 'guesslens *********** nloc', nloc, ' ndata', ndata
* no data
	if( ndata .le. 0 ) return

* debugging plot
c	verbose = .true.
	plots = .false.
c	plots = .true.
* default values
  100	t0 = 0.
	te = 1.
	y0 = 1.
	a0 = atotl( y0 )
	smag0 = 0.
* flux scale factor
	addmag = 0.
* blend flux
	b0 = 0.
* error bar fudge factors
	sigmag0 = 0.
	erbscl = 1.
      do i = 1, nloc
	fs_l(i) = 1.
	fb_l(i) = 0.
	sigfs_l(i) = 0.
	sigfb_l(i) = 0.
	apiv_l(i) = 1.
	addmag_l(i) = addmag
	b0_l(i) = b0
	sigmag0_l(i) = sigmag0
	erbscl_l(i) = erbscl
	syserr_l(i) = syserr
	avglogf_l(i) = avglogf
	siglogf_l(i) = siglogf
      end do
	call subsidiaries
* report
	write(*,*) 'guesslens default parameters:'
	loc = 1
	write(*,*) 'loc', loc, ' nloc', nloc
	call showpar

* data from first file
	ndat = ndat_l( 1 )
      if( ndat .le. 0 ) then
	write(*,*) '** WARNING: No data.'
      else

* report range of observed mags
	call mnmx( ndat, t_t, tmn, tmx )
	call mnmx( ndat, datmag_t, dmn, dmx )
	call mnmx( ndat, sigmag_t, smn, smx )
	datmed = xmedian( ndat, datmag_t )
	write(*,*) 'observed times:', ndat, tmn, tmx
	write(*,*) 'observed mags:', ndat, dmn, dmx, ' med', datmed
	write(*,*) 'observed sigs:', ndat, smn, smx

      if( plots ) then
	write(*,*)
	write(*,*) ' ** guesslens test plot **'
	write(*,*)
* open
	call pgstart( 1, 1, new )
	call pgkolors
	csize = 1.5
	call pgsch( csize )
* window
	xmn = tmn
	xmx = tmx
	add = max( 0.1, 0.05 * abs( dmx - dmn ) )
	ymn = dmx + add
	ymx = dmn - add
	call pgsci( kolor_window )
	call pgenv( xmn, xmx, ymn, ymx, 0, 0 )
* fiducials
	call pgsls( 2 )
	call pgmove( xmn, datmed )
	call pgdraw( xmx, datmed )
	label = ' '
	call append_f( label, datmed, '(f10.2)' )
	label = ' ' // label(:len1(label))
	call pgtext( xmx, datmed, label( : len1(label) ) )
	call pgsls( 1 )
* label
	xlabel = 't'
	ylabel = 'mag'
	title = 'guesslens'
	call title_event( title )
	call pgsci( kolor_label )
	call pglabel( xlabel, ylabel, title )
c	call pgsci( kolor_sigma )
c	call pgerrb( 2, ndat, t_t, datmag_t, sigmag_t, 1. )
c	call pgerrb( 4, ndat, t_t, datmag_t, sigmag_t, 1. )
	call pgsci( kolor_data )
	call pgpoint( ndat, t_t, datmag_t, 20 )
      end if

* optimal average observed mags
	call optavg( ndat, datmag_t, sigmag_t, avgmag, sigmag )

* baseline mag
	basemag = datmed
	basemag = avgmag

* badness of fit
      do k=1,3
	if( k .eq. 1 ) b = avgmag
	if( k .eq. 2 ) b = datmed
	if( k .eq. 3 ) b = basemag
	sum0 = 0.d0
      do i=1,ndat
	add = datmag_t(i) - b
	sum0 = sum0 + add * add
      end do
	chi2red = sum0 / max(1,ndat-1)
      if( k .eq. 1 ) then
	write(*,*) 'optimal average mag:', avgmag, '+/-', sigmag	
	write(*,*) 'Chi2=', sum0, ' /(', ndat, '- 1 ) =', chi2red
      else if( k .eq. 2 ) then
	write(*,*) 'median mag:', datmed
	write(*,*) 'Chi2=', sum0, ' /(', ndat, '- 1 ) =', chi2red
      else
	write(*,*) 'baseline mag:', basemag
	write(*,*) 'Chi2=', sum0, ' /(', ndat, '- 1 ) =', chi2red
      end if
      end do

* tsig grid
	tsiglog1 = 1./3.
	tsiglog2 = 2.0
	dtsiglog = 1./12.
	ntsig = nint( abs( tsiglog2 - tsiglog1 ) / abs( dtsiglog ) )
	ntsig = max( 2, ntsig )
	dtsiglog = abs( tsiglog2 - tsiglog1 ) / max( 1, ntsig - 1 )
	write(*,*) 'tsig grid:', ntsig,
     &	10.**tsiglog1, 10.**tsiglog2, ' d(logtsig)', dtsiglog
	bad = 1.e20
	best = 1.1 * bad
      do itsig=1,ntsig
	p = float(itsig-1) / max( 1, ntsig - 1 )
	tsiglog = tsiglog1 * (1.-p) + tsiglog2 * p
	tsig = 10.**tsiglog
	write(*,*) '-----', itsig, ' of', ntsig, ' tsig=', tsig, ' -----'

* t_0 grid
	dt0 = tsig * 0.1
	dtrange = abs( t_t(ndat) - t_t(1) )
	nt0 = nint( dtrange / abs( dt0 ) )
	nt0 = max( 2, nt0 )
	dt0 = dtrange / max( 1, nt0 - 1 )
	write(*,*) 't0 grid:', nt0, t_t(1), t_t( ndat ), ' d(t0)', dt0
	nodata = 0
	negmag = 0
	good = 1.1 * bad
      do it0 = 1, nt0
	p = float( it0 - 1 ) / max( 1, nt0 - 1 )
	t0 = t_t( 1 ) * (1.-p) + t_t( ndat ) * p

* gaussian pattern
      do i = 1, ndat
	g = 0.
	e = ( t_t(i) - t0 ) / tsig
      if( abs(e) .le. 6. ) then
	g = exp( - 0.5 * e * e )
      end if
	plot1(i) = g
      end do
	call optavg( ndat, plot1, sigmag_t, gavg, gsig )
* no support
      if( gavg .eq. 0. ) then
	nodata = nodata + 1
      if( verbose ) then
	write(*,*) itsig, it0, ' tsig', tsig, ' t0', t0, ' no support.    :('
      end if
* orthogonal pattern
      else
      do i=1,ndat
	plot1(i) = plot1(i) - gavg
	plot2(i) = basemag - datmag_t(i)
      end do
* scale pattern to fit
	call optscl( ndat, plot2, sigmag_t, plot1, amag, amagsig )
* negative magnification
      if( amag .lt. 0. ) then
	negmag = negmag + 1
      if( verbose ) then
	write(*,*) itsig, it0, ' tsig', tsig, ' t0', t0
     &		, ' Amag=(', amag, ' +/-', amagsig, ') NEGATIVE!    :O'
      end if
* badness of fit
      else
	sum = 0.d0
      do i=1,ndat
	add = plot2(i) - amag * plot1(i)
	sum = sum + add * add
      end do
	dchi2 = sum - sum0
	chi2red = sum / max( 1, ndat-2 )

* baseline mag
	s = basemag + gavg * amag
* peak magnification
	a = 10.**( 0.4 * amag )
* impact parameter
	y = ainv( a )
* set te to match half-way between peak and baseline magnitude
	th = thalf( a )
	ahalf = ( a + 1. ) / 2.
	ghalf = alog10( ahalf ) / alog10( a )
	fac = sqrt( - 2. * alog( ghalf ) )
	te = tsig * fac / th
* report
c	write(*,*) 'A0', a, ' t_1/2', th, ' fac', fac
c	write(*,*) 'ADJUST te:', te, ' *', fac, ' /', th, ' =>', te*fac/th 

* prior on te
	prior = 1.
      if( siglogte .gt. 0. ) then
	eta = ( alog10( te ) - avglogte ) / siglogte
	prior = exp( -0.5 * eta * eta )
      end if
	prlog = alog10( prior )
c	write(*,*) 'Prior( te=', te, ' ) =', prior, ' log(Pr)', prlog

* posterior = prior * likelihood
c	plog = prlog + alog10( abs( dtsiglog * dt0 ) )
c     &		+ alog10( exp( 1. ) ) * ( -0.5 * dchi2 )

	plog = prlog + alog10( exp( 1. ) ) * ( -0.5 * dchi2 )

* report
      if( verbose ) then
	write(*,*) itsig, it0, ' tsig', tsig, ' t0', t0
     &		, ' Amag=(', amag, ' +/-', amagsig, ')'
	write(*,*) ' Chi2=', sum, '/(',ndat,'-2)=', chi2red
     &	, ' Dchi2', dchi2
      end if

* initialize good and best
	test = dchi2
	test = - plog
      if( good .ge. bad ) then
	good = max( test, bad )
c	write(*,*) 'good set to', good
      if( best .ge. bad ) then
	best = max( test, bad )
c	write(*,*) 'best set to', best
      end if
      end if

* local best?
      if( test .le. good ) then
	good = test
	goodchi2 = dchi2
	goodlogpr = prlog
	goodlogp = plog
	t0g = t0
	teg = te
	s0g = s
	a0g = a
	y0g = y
      if( verbose ) then
	write(*,*) '** KEEP THIS ONE.  IT IS THE BEST YET !   :)'
      end if
      end if

      end if
      end if

* next t0
      end do

* report
      if( nodata .gt. 0 ) then
	percent = 100.* float( nodata ) / max( 1, nt0 )
	write(*,*) 'No data near', nodata, ' of', nt0
     &		, '=', nint(percent), ' % of t0 values.'
      end if
      if( negmag .gt. 0 ) then
	percent = 100. * float( negmag ) / max( 1, nt0 )
	write(*,*) 'Negative mag for', negmag, ' of', nt0
     &		, '=', nint(percent), ' % of t0 values.'
      end if

* report best for this tsig
	write(*,*) ' Local best     Log(Pr)', goodlogpr
     &		, '     Dchi2', goodchi2, ' Log(P)', goodlogp
	write(*,*) ' t0', t0g, ' tsig', tsig, ' te', teg
	write(*,*) ' m0', s0g, ' A', a0g, ' y0', y0g

* global best?
      if( good .le. best ) then
	best = good
	bestchi2 = goodchi2
	bestlogpr = goodlogpr
	bestlogp = goodlogp
	t0best = t0g	
	tsigbest = tsig
	tebest = teg
	smag0 = s0g
	a0 = a0g
	y0 = y0g
c      if( verbose ) then
	write(*,*) '** KEEP THIS ONE.  IT IS THE BEST YET !   :)'
c      end if
      end if
* report global best
	write(*,*) 'GLOBAL BEST     Log(Pr)', bestlogpr
     &		, '     Dchi2', bestchi2, ' Log(P)', bestlogp
	write(*,*) ' t0', t0best, ' tsig', tsigbest, ' te', tebest
	write(*,*) ' m0', smag0, ' A', a0, ' y0', y0


* diagnostic plot
      if( plots ) then
	np = min( maxplot, 1000 )
	nloop = 1
	if( itsig .eq. ntsig ) nloop = 2
      do loop=1,nloop
      do i = 1, np
	p = (i-1.)/max(1,np-1)
	t = xmn * (1.-p) + xmx * p
	g = 0.
* local fit (this tsig)
      if( loop .eq. 1 ) then
	cen = t0g
	sig = tsig
	s = s0g
	a = a0g

* global fit (last tsig)
      else
	cen = t0best
	sig = tsigbest
	s = smag0
	a = a0
      end if
	e = ( t - cen ) / sig
      if( abs(e) .le. 6. ) then
	g = exp( - 0.5 * e * e )
      end if
	plot1(i) = t
	plot2(i) = s - 2.5 * alog10( a ) * g
      end do
	call pgsci( loop )
	if( loop .gt. 1 ) call pgslw( 6 )
	call pgline( np, plot1, plot2 )	
	call pgslw( 1 )
	call pgmove( t0best, s )
	call pgdraw( t0best, s - 2.5 * alog10( a ) )

* next loop
      end do
	call pgslw( 1 )
      end if
* end of diagnostic plot

* next tsig
      end do

* adopt global best
	tsig = tsigbest
	te = tebest
	t0 = t0best
	hjd0 = mjd0 + t0

* boost a0 to match median mag if fainter than optavg mag
	a0g = a0
	y0g = y0
	s0g = smag0
	teg = te
	write(*,*) 'med mag', datmed, ' opt avg mag', smag0
      if( datmed .gt. smag0 ) then
	afac = 10. ** ( 0.4 * ( datmed - smag0 ) )
	a0 = a0 * afac
	y0 = ainv( a0 )
	tfac = thalf( a0g ) / thalf( a0 )
	te = te * tfac
	smag0 = datmed
	write(*,*) 'm0 =', s0g, ' => median mag', smag0
	write(*,*) 'A0 =', a0g, ' x', afac, ' =', a0
	write(*,*) 'u0 =', y0g, ' =>', y0
	write(*,*) 'te =', teg, ' x', tfac, ' =', te
      end if

* adjust te to match half-way between peak and baseline magnitude
	th = thalf( a0 )
* no need to do this because now done inside the tsig loop
      if( .false. ) then
c      if( .true. ) then
	ahalf = ( a0 + 1. ) / 2.
	ghalf = alog10( ahalf ) / alog10( a0 )
	fac = sqrt( - 2. * alog( ghalf ) )
* report
	write(*,*) 'A0', a0, ' t_1/2', th, ' fac', fac
	write(*,*) 'ADJUST te:', te, ' *', fac, ' /', th, ' =>', te*fac/th 
* apply
      if( fac .gt. 0. .and. th .gt. 0. ) then
	te = te * fac / th 
      else
	write(*,*) '** ERROR te correction FAILED :('
      end if
      end if

* report
      if( plots ) then
	call pgsci( kolor_label )
	title = 't\d0\u ='
	call append_f( title, t0, '(f15.2)' )
	call append( title, '\gs\dt\u' )
	call append_n( title, tsig, 3 )
	call append( title, 'A\d0\u' )
	call append_n( title, a0g, 3 )
      if( a0g .ne. a0 ) then
	call append( title, '=>' )
	call append_n( title, a0, 3 )
      end if
	call pgmtxt( 't', 0.5, 0.5, 0.5, title(:len1(title)) )
	title = 't\dE\u'
	call append_n( title, te, 3 )
	call append( title, 't\d1/2\u' )
	call append_n( title, te * th, 3 )
	call pgmtxt( 't', -1.5, 0.5, 0.5, title(:len1(title)) )

* plot final gaussian model
      do i = 1, np
	t = plot1(i)
	g = 0.
	cen = t0
	sig = tsigbest
	s = smag0
	a = a0
	e = ( t - cen ) / sig
      if( abs(e) .le. 6. ) then
	g = exp( - 0.5 * e * e )
      end if
	plot2(i) = s - 2.5 * alog10( a ) * g
      end do
	call pgsci( 3 )
	call pgslw( 6 )
	call pgline( np, plot1, plot2 )	
	call pgslw( 1 )
	call pgmove( t0best, s )
	call pgdraw( t0best, s - 2.5 * alog10( a ) )

      end if
	
* adjust site-dependent parameters
      do i=1,nloc
	fb_l(i) = 0.
	fs_l(i) = 1.
      end do
* report
c	call showpar
c	call showsitepar

* fit source and blend fluxes
	npfit = 1
c	write(*,*) '** fitfsfb Npfit', npfit
	call fitfsfb
* report
c	write(*,*) '** showsitepar'
c	call showsitepar
      end if
* final report
	write(*,*) 'guesslens final parameters:'
	call showpar
	write(*,*) '*********************************'

      if( plots ) then
	call pgstop
	if( pgagain() ) goto 100
      end if

	return
	end
