

*----------------------
	integer*4 function imclose( utest, atest )
* Is planet "close" to an image ?
* Input:
*	utest	r4 distance of planet from lens (R_E)
*	atest	r4 azimuth of planet (radians)
* Output:
*	imclose	i4 1 if close, 0 if not close to an image
* 2002 Nov Keith Horne @ St-Andrews
* 2002 Nov KDH @ St-And - use plot1,plot2 to sort for speed
* 2009 Jul KDH @ St-A - trap q<0, real*8 sum
	include 'plens.inc'
	real*8 sum
	data qold/-1./

* trap invalid input
      if( qpl .le. 0. ) then
	write(*,*) 'imclose(', utest, atest, ' ) for q=', qpl
	write(*,*) '** ERROR invalid q'
	imclose = 1
	return
      end if

      if( qpl .ne. qold ) then
	repl = sqrt( qpl )
	qold = qpl
      end if

* data points
	sum = 0.d0
	nsum = 0
      do i = 1, ndata

* assume these are all available
	emag = erbmag_t( i )
* image positon
	xp = xp_t( i )
	yp = yp_t( i )
	xm = xm_t( i )
	ym = ym_t( i )
	up = up_t( i )
	um = um_t( i )
* magnification
	ap = ap_t( i )
	am = am_t( i )

      do im = 1, 2
* major image
      if( im .eq. 1 ) then
	uim = abs( up )
	aim = abs( ap )
	area = aim + aim - 1.
* minor image
      else
	uim = abs( um )
	aim = abs( am )
	area = aim + aim
      end if

* COULD BE SMALLER IF WE ACCOUNT FOR BLENDING
	dza = area * qpl / emag
	dzw = dza / ( uim * uim )
	bigdlnu = sqrt( dzw )

* close in logr ?
	extra = 1.5
	dlnu = alog( utest / uim )
	bigu = extra * bigdlnu
      if( abs( dlnu ) .le. bigu ) then

* close in azimuth ?
	bigdaz = bigdlnu
      if( im .eq. 1 ) then
	call xy2a( xp, yp, azim )
      else
	call xy2a( xm, ym, azim )
	bigdaz = bigdaz + repl
      end if
	biga = extra * bigdaz

	daz = atest - azim 
      do while( daz .gt. pi ) 
	daz = daz - twopi
      end do
      do while( daz .lt. - pi )
	daz = daz + twopi
      end do

      if( abs( daz ) .le. biga ) then
* sum lorentzians (WHY?)
	eu = dlnu / bigdlnu
	ea = daz / bigdaz
	ee = eu * eu + ea * ea
* add drops from 2 to 1 at the boundary and to 0 beyond.
	add = 2. / ( 1. + ee )
	sum = sum + add
	nsum = nsum + 1

* debug report
      if( .false. ) then
      if( sum .gt. 0.1 ) then
	call ua2xy( utest, atest, xtest, ytest )
	call ua2xy( uim, azim, xim, yim )
	write(*,*) 'PIXEL ua', utest, rad2deg(atest), ' xy', xtest, ytest
	write(*,*) 'IMAGE ua', uim, rad2deg(azim), ' xy', xim, yim
	write(*,*) 'sig(', i, ' )=', emag, ' im', im, ' A', aim, ' W', dzw
	write(*,*) 'ln(', utest, ' /', uim, ' )=', dlnu, ' /', bigu, ' =', dlnu/bigu
	write(*,*) '  (', atest, ' -', azim, ' )=', daz, ' /', biga, ' =', daz/biga
	write(*,*) 'eu', eu, ' ea', ea, ' ee', ee, ' add', add, ' sum', sum
      end if
      end if

* this pixel has enough images close to it.
      if( sum .gt. 1.0 ) then
	imclose = 1
      if( .false. ) then
	write(*,*) 'IMCLOSE nsum', nsum, ' sum(', i, ' )=', sum
	reply = ' '
	call inq( '<CR> to continue', reply, reply )
      end if
	return
      end if


      end if
      end if
* next image
      end do
* next data point
      end do
* report

	imclose = 0
	return
	end
