
*-------------------------------------------------
	subroutine lcrebin_l( l )
* rebin site l lightcurve data (baseline data only)
* Input:
*	l	i4 site number
*	binday	r4 max bin size in years
*	binyear	r4 bin data older than this many years
*	binte	r4 bin data outside this times t_E
*
* 2008 July Keith Horne @ St.And 
* 2009 May KDH @ St.A rebin only if near median mag
* 2009 May KDH @ St.A binday, binyear, binte from plens.inc
* 2009 Jul KDH @ St-A  - t0 => hjd0
	include 'plens.inc'
	real*8 hjdfirst, hjdlast, sum
	year = 365.25
	t0 = hjd0 - mjd0

	write(*,*) '** lcrebin(', l, binday, binyear, binte, ' )'
	write(*,*) 'a0', a0, ' t0', t0, ' te', te, ' t1/2', te*thalf(a0)

* trap input errors
      if( l .lt. 1 .or. l .gt. nloc ) then
	write(*,*) '** ERROR in lcrebin.  Site ', l, ' outside 1 -', nloc
	return
      end if

	site = site_l( l )
	ifirst = it1_l( l )
	ilast = it2_l( l )
	ndat = ndat_l( l )
	nold = ilast - ifirst + 1

	hjdlast = hjd_t( ilast )
	hjdfirst = hjd_t( ifirst )

	write(*,*)
	write(*,*) 'site', l, ' of', nloc
     &		, ' first', ifirst, ' last', ilast
     &		, ' ndat', ndat, ' of', ndata

      if( ndat .le. 0. ) then
	write(*,*) '** WARNING: No data to rebin.'
	return
      end if

	write(*,*) 'Days/bin:', binday
      if( binday .le. 0. ) then
	write(*,*) '** WARNING: binday must be positive.'
	return
      end if

* age clip
	ageclip = binyear * year
	years = ( hjdlast - hjdfirst ) / year
	write(*,*) 'Age limit', binyear, ' years'
	write(*,*) 'HJD range', hjdfirst, hjdlast, ' years', years

* X clip
	xclip = binte
	t_half = te * thalf( a0 )
	tfirst = hjdfirst - mjd0
	tlast = hjdlast - mjd0
	write(*,*) 'X limit', binte, ' t_E'
	write(*,*) 'X range', (tfirst-t0)/te , (tlast-t0)/te

* mag range - all data
	n = 0
      do i=ifirst, ilast
      if( sigflx_t(i) .gt. 0. ) then
	n = n + 1
	plot2(n) = datmag_t(i)
      end if
      end do
	call avgrms( n, plot2, datavg, datrms )
	call medmad( n, plot2, datmed, datmad )
	f0 = fs_l(l) + fb_l(l)
	basemag = tomag( f0 )

* mag range - baseline
	n = 0
      do i=ifirst, ilast
      if( sigmag_t( i ) .gt. 0. ) then
	age = hjdlast - hjd_t(i)
	x = ( hjd_t( i ) - hjd0 ) / te
      if( age .gt. ageclip .and. abs( x ) .gt. xclip ) then
	n = n + 1
	plot2(n) = datmag_t(i) - basemag
      end if
      end if
      end do
	ipct = n * 100. / ndat
      if( n .ge. 1 ) then
	call avgrms( n, plot2, baseavg, baserms )
	call medmad( n, plot2, basemed, basemad )
	baseavg = baseavg + basemag
	basemed = basemed + basemag
      end if

* report
	write(*,*) 'Site', l, ' (', ifirst, ilast, ' )=', ndat, ' ', site(:len1(site))
	write(*,*) 'Baseline fit', basemag, ' data:', n, ' =', ipct, ' %'
	write(*,*) 'All data avg', datavg, ' rms', datrms, ' bias', datavg-basemag
	write(*,*) '         med', datmed, ' mad', datmad, ' bias', datmed-basemag
      if( n .gt. 1 ) then
	write(*,*) 'Baseline avg', baseavg, ' rms', baserms, ' bias', baseavg-basemag
	write(*,*) '         med', basemed, ' mad', basemad, ' bias', basemed-basemag
      end if

* mag clip
      if( n .gt. 1 .and. basemad .gt. 0. ) then
	binmag = basemed
	binclip = basemad
	bias = binmag - basemag
	write(*,*) 'Using Baseline med', binmag, ' mad', binclip, ' bias', bias
      else
	binmag = datmed
	binclip = datmad
	bias = binmag - basemag
	write(*,*) 'Using All data med', binmag, ' mad', binclip, ' bias', bias
      end if

	write(*,*) 'npfit', npfit
      if( npfit .ge. 4 .and. bias .gt. binclip ) then
	binmag = basemag
	write(*,*) 'Bias too large.'
	write(*,*) 'Switch to baseline fit ', basemag, ' mad', binclip
      end if
	binclip = binclip

* flux clip
	binflx = binmag
	flxclip = binclip
* Try this
	binflx = basemag
	flxclip = 0.
	write(*,*) 'Using baseline fit mag', basemag, ' sigma clip', binsigclip

	call mag_to_flx( binflx, flxclip )
	n = 0
      do i=ifirst, ilast
	d = datflx_t(i)
	s = sigflx_t(i)
	v = s * s + flxclip * flxclip
	eps = ( d - binflx ) / sqrt( s * s + flxclip * flxclip )
	ok_t(i) =  abs( eps ) .lt. binsigclip
	n = n + 1
      end do

* summary
	write(*,*) 'Rebin mag range', binmag - binclip, binmag + binclip
	write(*,*) '      flx range', binflx - flxclip, binflx + flxclip
	write(*,*) '      outside', xclip, ' x t_E =', xclip * te, ' days.'
	write(*,*) '      before', hjdlast - binyear * year, ' =', ageclip, ' days.'

* count new bins
	new = 0
* start of bin
	i1 = ifirst
	next = i1
      do while( i1 .le. ilast )

* shorten bins to 0 for age < ageclip
	age = hjdlast - hjd_t( i1 )
	tbin = binday * min( 1., max( 0., age - ageclip ) )

* shorten bins to 0 for abs(x) < xclip
      if( xclip .gt. 0. ) then
	x = ( hjd_t( i1 ) - hjd0 ) / te
	tbin = tbin * min( 1., max( 0., abs( x ) - xclip ) )
      end if

* find end of bin
	i2 = i1 + 1
      if( ok_t( i1 ) ) then
      do while( i2 .le. ilast .and. ok_t( i2 )
     &		.and. hjd_t(i2) - hjd_t(i1) .lt. tbin
     &	)
	i2 = i2 + 1
      end do
      end if
	i2 = i2 - 1

* number of points to average
	nbin = i2 - i1 + 1
* HJDs
	hjd = ( hjd_t(i2) + hjd_t(i1) ) / 2.d0
	tspan = hjd_t(i2) - hjd_t(i1)
      do i=i1,i2
	plot1( i ) = hjd_t( i ) - hjd
      end do
	call optavg( nbin, plot1( i1 ), sigflx_t( i1 ), t, sig )
	hjd = hjd + dble(t)
* fluxes
	call optavg( nbin, datflx_t( i1 ), sigflx_t( i1 ), avgflx, sigflx )

* sanity check
      if( .false. ) then
      if( nbin .eq. 2 ) then
	write(*,*) i1, i2, ' hjd', hjd_t(i1), hjd_t(i2), hjd
	write(*,*) i1, i2, ' hjd', plot1(i1), plot2(i2), t
	write(*,*) i1, i2, ' dat', datflx_t(i1), datflx_t(i2)
	write(*,*) i1, i2, ' sig', sigflx_t(i1), sigflx_t(i2)
	write(*,*) ' avg', avgflx, ' sig', sigflx
      end if
      end if

* mags ( compute later from fluxes )
c	call optavg( nbin, datmag_t( i1 ), sigmag_t( i1 ), avg, sig )
* seeing
	call optavg( nbin, see_t( i1 ), sigflx_t( i1 ), see, sig )
* sky
	call optavg( nbin, sky_t( i1 ), sigflx_t( i1 ), sky, sig )
* exposure time
	sum = 0.d0
      do i=i1,i2
	if( sigflx_t(i) .gt. 0. ) sum = sum + dwell_t(i)
      end do
	dwell = sum
* stow
	hjd_t( next ) = hjd
	dwell_t( next ) = dwell
	datflx_t( next ) = avgflx
	sigflx_t( next ) = sigflx

* mags ( compute later from fluxes )
c	datmag_t( next ) = avg
c	sigmag_t( next ) = sig

	see_t( next ) = see
	sky_t( next ) = sky
	clip_t( next ) = 1.
	new = new + 1
	next = next + 1

* report
      if( nbin .gt. 1 ) then
	title = 'bin'
	call append_i( title, new )
	call append( title, '(' )
	call append_i( title, i1 )
	call append_i( title, i2 )
	call append( title, ')=' )
	call append_i( title, nbin )
* hjd
	call append( title, 'hjd' )
	t = hjd - mjd0
	call append_f( title, t, '(f15.1)' )
* dt
	call append( title, 'dt' )
	call append_f( title, tspan, '(f15.1)' )
* x
	call append( title, 'x' )
	x = ( hjd - hjd0 ) / te
	call append_f( title, x, '(f15.1)' )
* age
	call append( title, 'age' )
	call append_f( title, age, '(f15.1)' )
* mag
	avg = avgflx
	sig = sigflx
	call flx_to_mag( avg, sig )
	call append( title, 'mag' )
	call append_f( title, avg, '(f15.3)' )
	call append( title, '+/-' )
	call append_f( title, sig, '(f15.3)' )

	write(*,*) title(:len1(title))
      end if

* next bin
	i1 = i2 + 1
      end do
	i2 = next - 1
* report
	compress = float( nold ) / max(1,new)
	ipct = new * 100. / nold
	write(*,*) 'Site', l, ' (', ifirst, ilast, ' )=>(', ifirst, i2, ' )'
     &		, ' ndat', nold, ' =>', new, ' ', site(:len1(site))
     &		, ' scrunch', compress
	ndat_l( l ) = new
	it2_l( l ) = i2

* sanity check
	inew = it2_l(l) - it1_l(l) + 1
      if( new .ne. inew ) then
	write(*,*) '** ERROR in lcrebin.', new, inew
	call die
      end if

* move forward lightcurves of other sites
      if( nloc .gt. l ) then
      do j = l+1, nloc
	site = site_l( j )
	i1 = it1_l( j )
	i2 = it2_l( j )
	ndatold = ndat_l( j )
	it1_l( j ) = next
      do i = i1, i2
	loc_t( next ) = loc_t( i )
	hjd_t( next ) = hjd_t( i )
	dwell_t( next ) = dwell_t( i )
	datflx_t( next ) = datflx_t( i )
	datmag_t( next ) = datmag_t( i )
	sigflx_t( next ) = sigflx_t( i )
	sigmag_t( next ) = sigmag_t( i )
	see_t( next ) = see_t( i )
	sky_t( next ) = sky_t( i )
	clip_t( next ) = clip_t( i )
	next = next + 1
      end do
	it2_l( j ) = next - 1
	ndat = it2_l(j) - it1_l(j) + 1
	write(*,*) 'Move Site', j, ' (', i1, i2, ' )=>(', it1_l(j), it2_l(j), ' )'
     &		, ' ndat', ndatold, ' =>', ndat, ' ', site(:len1(site))

* next site
      end do
      end if

* number of data points
	ndataold = ndata
	ndata = next - 1
	ipct = ndata * 100. / ndataold
	compress = float( ndataold ) / max(1,ndata)
	write(*,*) 'Total data', ndataold, '=>', ndata, ' scrunch', compress

	return
	end
