*------------------------------
	subroutine rplot
* plot magnitude residuals
* 1998 Aug Keith Horne @ St-Andrews
* 2003 Jul KDH @ St-And - cormag, pgxys, vs_mag
* 2003 Jul KDH @ St-And - ditch and restore planet
* 2003 Jul KDH @ St-And - re-fit with no planet to evaluate delchi2
* 2003 Nov KDH @ St-And - cormag, erbmag
* 2006 Feb KDH @ St-And - timeaxis
* 2006 Mar KDH @ St-And - marknow
* 2008 Jun KDH @ StA - asymmetric error bars
* 2009 Jul KDH @ StA  - t0 => hjd0, premag,flx(t)->premag,flx(hjd)
* 2009 Jul KDH @ StA  - fix crash on ndata=0
* 2010 Apr KDH @ StA  - fix plot of residuals with/wo planet
	include 'plens.inc'
	write(*,*) 'rplot **************** nloc', nloc, ' ndata', ndata
	t0 = hjd0 - mjd0

      if( ndata .gt. 0 ) then
* report chi2 with and without the planet
* stow with-planet fit
	call stowall
	write(*,*) 'With-planet fit all parameters stowed.'
* ditch the planet and refit
	call noplanetfit
* chi^2 with no planet
	call chi2calc( bof, ndf )
* save no-planet fit
	call savepar( 10 )
	write(*,*) 'No-planet fit', 10, ' parameters saved.'
* reduced chi^2
	np_nopl = min( 7, npfit )
	ndof = max( 0, ndata - np_nopl )
	chi2red_nopl = bof / max( 1, ndof )
* report
	write(*,*) 'NO-PLANET -----'
     &	, ' Chi^2', bof, '/(', ndata, '-', np_nopl, ' )=', chi2red_nopl
	call showpar

* restore planet fit
	call restoreall

* chi^2 with planet
	call chi2calc( chi2min, ndf )
* delta chi^2
	dchi2 = bof - chi2min
* reduced chi^2
	ndof = max( 0, ndata - npfit )
	chi2red = chi2min / max( 1, ndof )
* report
	write(*,*) 'WITH-PLANET ---'
     &	, ' Chi^2', chi2min, '/(', ndata, '-', np_nopl, ' )=', chi2red
     &	, ' DelChi^2', dchi2

	call showpar
      if( dchi2 .lt. 0. ) then
	write(*,*) '** WARNING: NO-PLANET FIT IS BETTER ?!  :-(' 
      end if
      end if

* time axis
	call timeaxis( xmn, xmx, xlabel )

* calculate mag residuals
	sum = 0.d0
	call xcalc
	bot = toflx( smag0 + 3. )
      do i=1,ndata
	hjd = hjd_t( i )
* predicted mag and flux
	pmag = premag( hjd )
	pflx = preflx( hjd )
* observed flux corrected to site 1 source and blend flux
	flx = corflx( i )
	sigflx = erbcorflx( i, pflx )
* ends of flux error bar
	fnt = flx - sigflx
	brt = flx + sigflx
* convert to magnitudes 
	cmag = tomag( max( bot, flx ) )
	emagp = tomag( max( bot, fnt ) ) - cmag
	emagm = cmag - tomag( max( bot, brt ) )
* stow
	plot1(i) = x_t(i)
	if( hjdaxis ) plot1(i) = hjd - mjd0
	plot2(i) = cmag - pmag
	plot3(i) = emagp
	plot4(i) = emagm
* sum
	eta = ( flx - pflx ) / sigflx
	sum = sum + eta * eta
      end do

* sanity check
	write(*,*) 'Chi2:', sum, ' DelChi2:', sum - chi2min
	write(*,*) 'Zero:', bof - sum, dchi2 - sum + chi2min

* window
	ymx = 0.05
	ymn = - ymx
      if( ndata .gt. 0 ) then
c	call mnmx( ndata, plot2, rmn, rmx )
	rmn = ymn
	rmx = ymx
      do i=1,ndata
      if( plot1(i) .ge. xmn .and. plot1(i) .le. xmx ) then
	rmn = min( plot2(i), rmn )
	rmx = max( plot2(i), rmx ) 
      end if
      end do
	add = 0.05
	ymn = min( ymn, rmn - add )
	ymx = max( ymx, rmx + add )
	ymx = min( ymx, 1.0 )
      end if

	write(*,*) 't0', t0, ' te', te, ' tbeg', tbeg, ' tend', tend
	write(*,*) 'x:', xmn, xmx, ' y:', ymn, ymx
	call pgsci( kolor_window )
	call pgenv( xmn, xmx, ymx, ymn, 0, 0 )

* horizontals
	call pgsci( kolor_model )
	call pgmove( xmn, 0. )
	call pgdraw( xmx, 0. )
* verticals
	call pgsls( 4 )
	call marktime( ymn, ymx )
	call pgsls( 1 )
* now
	call pgsci( kolor_now )
	call marknow( ymn, ymx )

* labels
	ylabel = 'residual (mag)'
      if( ndata .eq. 0 ) then
	title = 'Magnitude Residuals'
      else
	title = ' '
	call title_event( title )
      end if
	call pgsci( kolor_label )
	call pglabel( xlabel, ylabel, title )
	call legend

* Bring external data to edge of window
      do i=1,ndata
	plot2(i) = max( ymn, min( ymx, plot2(i) ) )
      end do
* residuals
	call pgxys( plot1, plot2, plot3, plot4 )



* residual lightcurve with/without planet
      if( qpl .gt. 0. ) then
	write(*,*) 'Residual lightcurve with/without planet q', qpl
	nplt = 1000
      do i=1,nplt
	part = float(i-1) / max(1,nplt-1)
	xxx = xmn + part * ( xmx - xmn )
	plot0(i) = xxx
	if( hjdaxis ) xxx = ( xxx - t0 ) / te
	x_t(i) = xxx
      end do

      do ipl=1,2
* restore noplanet fit
      if( ipl .eq. 1 ) then
	write(*,*) 'Restore no-planet fit:'
	call restorepar( 10 )
* restore planet
      else
	write(*,*) 'Restore with-planet fit:'
	call restoreall
      end if
	call showpar

* lightcurve
	call lcalc( nplt, x_t, y0 )
* report
c	write(*,*) 'Nplt', nplt, ' y0', y0, ' x', x_t(1), x_t(nplt)
	call mnmx( nplt, a_t, alo, ahi )
c	write(*,*) 'Magnification A', alo, ahi
c      do i=1,nplt, max(1,nplt/10)
c	write(*,*) i, ' x', x_t(i), ' A', a_t(i)
c      end do

* no-planet lightcurve
      if( ipl .eq. 1 ) then
      do i=1,nplt
	plot1(i) = smag0 + blendmag( a_t(i), b0 )
      end do
	call mnmx( nplt, plot1, pmn, pmx )
	write(*,*) 'No-planet lightcurve', pmn, pmx
* with-planet residual lightcurve
      else
      do i=1,nplt
	plot1(i) = ( smag0 + blendmag( a_t(i), b0 ) ) - plot1(i)
      end do
	call mnmx( nplt, plot1, pmn, pmx )
	write(*,*) 'With-planet residual lightcurve', pmn, pmx

	call pgsci( kolor_model )
	call pgline( nplt, plot0, plot1 )
      end if

* next ipl
      end do
      end if

	return
	end
