*------------------------------
	subroutine title_chi2( title )
* add Chi^2 to title
* recalculate Chi^2 for fits 4,5,6,7 only if required.
*
* 1998 Aug Keith Horne @ St-Andrews
* 2003 Jul KDH @ St-And -- use append
* 2003 Jul KDH @ St-And -- skip needless calculations
* 2004 Jun KDH @ St-And -- stow/restore values if re-fitting
* 2007 Mar KDH @ Goettingen -- tiny thresholds for re-calculating
* 2007 Apr KDH @ St-And -- 4,5,6,7
* 2008 Jun KDH @ St-And -- use twin for t0 step
* 2008 Jun KDH @ St-And -- stow site-dependent parameters
* 2009 Jul KDH @ St-A -- excuse
* 2010 Mar KDH @ LCO -- restoreall on 4->5 param fits
	include 'plens.inc'
* saved parameter values (from this call)
	real*8 savehjd0
	real*4 save( maxparm )
* parameter change thresholds
	real*4 tiny( maxparm )
* Chi^2/N values from previous fits
	real*4 oldchi2red( maxparm )
	character*128 excuse
	logical firstcall/.true./
	logical changes
	data k0, kt, km, ku, ks, kf, kb, kx, ky, kq, kn, kd
     &	/     1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12 /
* do nothing if no data
	if( ndata .le. 0 ) return
c	write(*,*) 'title_chi2 enter nalways', nalways, ' nperloc', nperloc

* first call
      if( firstcall ) then
* number of parameters always stowed
	nalways = max( k0, kt, km, ku, ks, kf, kb, kx, ky, kq, kn, kd )
* where to stow location-specific parameters
	ib0 = 1
	isigmag0 = 2
	ierbscl = 3
	iaddmag = 4
	isitefac = 5
	iaddflx = 6
* number of location-specific parameters
	nperloc = max( ib0, isigmag0, ierbscl, 
     &		iaddmag, isitefac, iaddflx )
c	write(*,*) 'title_chi2 firstcall.'
c	write(*,*) 'nalways', nalways, ' nperloc', nperloc
	changes = .true.
	firstcall = .false.
      else
c	write(*,*) 'title_chi2 not firstcall.'
c	write(*,*) 'nalways', nalways, ' nperloc', nperloc
* change thresholds requiring recalculation
	eps = 1.e-4
	eps = 3.e-4
	th = thalf( a0 )
	twin = max( 0.1, min( 1., th ) ) * te
	tiny( k0 ) = max( eps, eps * twin )
	tiny( kt ) = max( eps, eps * te )
	tiny( km ) = eps
	tiny( ku ) = max( eps, eps * y0 )
	tiny( ks ) = max( 1.e-3, 1.e-2 * sigmag0 )
	tiny( kf ) = max( 1.e-3, 1.e-2 * erbscl )
	tiny( kb ) = max( eps, eps * abs( b0 ) )
	rpl = sqrt( xpl * xpl + ypl * ypl )
	tiny( kx ) = max( eps, eps * sqrt( rpl ) )
	tiny( ky ) = tiny( kx )
	tiny( kq ) = max( 1.e-6, eps * abs( qpl ) )

* changes since previous call ?
	t0 = hjd0 - mjd0
	changes = 
     &       abs( save( k0 ) - t0 ) .gt. tiny( k0 )
     &	.or. abs( save( kt ) - te ) .gt. tiny( kt )
     &	.or. abs( save( km ) - smag0 ) .gt. tiny( km )
     &	.or. abs( save( ku ) - y0 ) .gt. tiny( ku )
     &	.or. abs( save( ks ) - sigmag0 ) .gt. tiny( ks )
     &	.or. abs( save( kf ) - erbscl ) .gt. tiny( kf )
     &	.or. abs( save( kb ) - b0 ) .gt. tiny( kb )
     &	.or. abs( save( kx ) - xpl ) .gt. tiny( kx )
     &	.or. abs( save( ky ) - ypl ) .gt. tiny( ky )
     &	.or. abs( save( kq ) - qpl ) .gt. tiny( kq )
     &	.or. nint( save( kn ) ) .ne. npfit
     &	.or. nint( save( kd ) ) .ne. ndata

c	write(*,*) 'title_chi2 ************', ndata, nint(save( kd )), ndfit, npfit

* report reason for re-computing
      if( changes ) then
	write(*,*) '--------- RE-CALCULATE Chi^2 BECAUSE:'

	if( nint( save( kn ) ) .ne. npfit )
     &		write(*,*) 'nfit', save( kn ), ' =>', npfit
	if( nint( save( kd ) ) .ne. ndata )
     &		write(*,*) 'ndat', save( kd ), ' =>', ndata

	kcheck = max( 0, k0, kt, km, ku, ks, kf, kb, kq, kx, ky )
      do k = 1, kcheck
	excuse = ' '
      if( k .eq. k0 ) then
	change = save( k0 ) - t0
	thresh = tiny( k0 )
	if( abs( change ) .gt. thresh )
     &		write(excuse,*) '  t0', save( k0 ), ' =>', t0

      else if( k .eq. kt ) then
	change = save( kt ) - te
	thresh = tiny( kt )
	if( abs( change ) .gt. thresh )
     &		write(excuse,*) '  te', save( kt ), ' =>', te

      else if( k .eq. km ) then
	change = save( km ) - smag0
	thresh = tiny( km )
	if( abs( change ) .gt. thresh )
     &		write(excuse,*) '  m0', save( km ), ' =>', smag0

      else if( k .eq. ku ) then
	change = save( ku ) - y0
	thresh = tiny( ku )
	if( abs( change ) .gt. thresh )
     &		write(excuse,*) '  y0', save( ku ), ' =>', y0

      else if( k .eq. ks ) then
	change = save( ks ) - sigmag0
	thresh = tiny( ks )
	if( abs( change ) .gt. thresh )
     &		write(excuse,*) '  sig0', save( ks ), ' =>', sigmag0

      else if( k .eq. kf ) then
	change = save( kf ) - erbscl
	thresh = tiny( kf )
	if( abs( change ) .gt. thresh )
     &		write(excuse,*) '  f', save( kf ), ' =>', erbscl

      else if( k .eq. kb ) then
	change = save( kb ) - b0
	thresh = tiny( kb )
	if( abs( change ) .gt. thresh )
     &		write(excuse,*) '  b', save( kb ), ' =>', b0

      else if( k .eq. kx ) then
	change = save( kx ) - xpl
	thresh = tiny( kx )
	if( abs( change ) .gt. thresh )
     &		write(excuse,*) '  x', save( kx ), ' =>', xpl

      else if( k .eq. ky ) then
	change = save( ky ) - ypl
	thresh = tiny( ky )
	if( abs( change ) .gt. thresh )
     &		write(excuse,*) '  y', save( ky ), ' =>', ypl

      else if( k .eq. kq ) then
	change = save( kq ) - qpl
	thresh = tiny( kq )
	if( abs( change ) .gt. thresh )
     &		write(excuse,*) '  q', save( kq ), ' =>', qpl
      end if

      if( excuse .ne. ' ' ) then
	call append( excuse, '(' )
	call append_n( excuse, change, 2 )
	call append( excuse, '/' )
	call append_n( excuse, thresh, 2 )
	call append( excuse, '=' )
	call append_n( excuse, change/thresh, 3 )
	call append( excuse, ')' )
	write(*,*) excuse(:len1(excuse))
      end if
      end do
      end if

      end if

* stow values that may change with fits
      if( changes ) then
	nstow = nalways + nperloc * nloc
      if( nstow .gt. maxparm ) then
	write(*,*) '** ERROR maxparm', maxparm, ' exceeded.'
	write(*,*) '**', nalways, ' +', nperloc, ' x', nloc, '  =', nstow
     &		, ' > maxparm=', maxparm
	call die
      end if

* calculate chi2
	call chi2calc( bof, ndf )
	bof = bof / max( 1, ndf - npfit )
* report
	write(*,*) '----------'
     &	, ' INITIAL CHI2/(', ndf, ' - ', npfit, ' ) = ', bof
* stow all parameters (to be restored at the end)
	call stowall
* save local parameters
	savehjd0 = hjd0
	t0 = hjd0 - mjd0
	save( k0 ) = t0
	save( kt ) = te
	save( km ) = smag0
	save( ku ) = y0
	save( ks ) = sigmag0
	save( kf ) = erbscl
	save( kb ) = b0
	save( kx ) = xpl
	save( ky ) = ypl
	save( kq ) = qpl
	save( kn ) = npfit
	save( kd ) = ndata

* stow site-dependent values
      if( .true.  ) then
c      if( .false. ) then
	last = nalways
      do i=1,nloc
	save( last + ib0 ) = b0_l( i )
	save( last + isigmag0 ) = sigmag0_l( i )
	save( last + ierbscl ) = erbscl_l( i )
	last = last + nperloc
      end do
      end if
c	write(*,*) 'title_chi2...', ndata, npfit, ' stow params:'
      else
c	write(*,*) 'title_chi2...', ndata, npfit, ' same params:'
      end if
c	call showpar

	if( ndfit .le. 0 ) ndfit = ndata

* chi2 for 4, 5, 6, 7-parameter fits
	npf1 = 4
	npf1 = min( 4, npfit )
	npf2 = max( npf1, min( 7, npfit ) )
      do npf = npf1, npf2

* re-fit
      if( changes ) then
      if( npf .ge. 4 ) then
c	write(*,*) 'title_chi2 refit parameters:', npf

* 4-parameter (no blending) fit
      if( npf .eq. 4 ) then
	call restoreall
	b0 = 0.
	sigmag0 = 0.
	erbscl = 1.
      do i=1,nloc
	b0_l( i ) = 0.
	sigmag0_l( i ) = 0.
	erbscl_l( i ) = 1.
      end do

* 5-prameter (with blending) fit
      else if( npf .eq. 5 ) then
	write(*,*) '     b0 =', b0, ' =>', save(kb)
	call restoreall
	b0 = save( kb )
	sigmag0 = 0.
	erbscl = 1.
	last = nalways
      do i=1,nloc
	e = b0_l(i)
	b0_l( i ) = save( last + ib0 )
	sigmag0_l( i ) = 0.
	erbscl_l( i ) = 1.
c	write(*,*) 'site', i, ' b0 =', e, ' =>', b0_l(i)
	last = last + nperloc
      end do
	
* 6-parameter (sigmag0) fit
      else if( npf .eq. 6 ) then
	write(*,*) '     sigmag0 =', sigmag0, ' =>', save(ks)
	sigmag0 = save( ks )
	last = nalways
      do i=1,nloc
	e = sigmag0_l(i)
	sigmag0_l( i ) = save( last + isigmag0 )
	last = last + nperloc
c	write(*,*) 'site', i, ' sigmag0 =', e, ' =>', sigmag0_l(i)
      end do

* 7-parmameter (erbscl) fit
      else if( npf .eq. 7 ) then
	write(*,*) '     erbscl =', erbscl, ' =>', save(kf)
	erbscl = save( kf )
	last = nalways
      do i=1,nloc
	e = erbscl_l(i)
	erbscl_l( i ) = save( last + ierbscl )
	last = last + nperloc
c	write(*,*) 'site', i, ' erbscl =', e, ' =>', erbscl_l(i)
      end do

      end if

* fit npf parameters (don't let fit1 alter npfit)
	npfitsave = npfit
	call fit1( nits, npf, bof )
	npfit = npfitsave
      end if

* calculate chi2
	call chi2calc( bof, ndf )
	bof = bof / max( 1, ndf - npf )
* do we need this ?
	ndfit = ndf
* report
	write(*,*) '----------'
     &	, ' STOW CHI2/(', ndf, ' - ', npf, ' ) = ', bof
* stow
	oldchi2red( max(1,npf) ) = bof

	if( npf .eq. 4 ) chi2_4 = bof
	if( npf .eq. 5 ) chi2_5 = bof
	if( npf .eq. 6 ) chi2_6 = bof
	if( npf .eq. 7 ) chi2_7 = bof

      end if

* fetch chi2 value (might be value from previous call)
	bof = oldchi2red( max(1,npf) )

      if( npf .eq. npf1 ) then
	l = len1( title )
	call append( title, '\gx\u2\d/(' )
	call append_i( title, ndfit )
	call append( title, '-' )
	call append_i( title, npfit )
	call append( title, ')=' )
	call nospace( title(l+2:) )
      else
	call append( title, '=>' )
      end if
	call append_f( title, bof, '(f11.2)' )
* next fit
      end do
	if( title(1:1) .eq. ' ' ) title = title(2:)
	call noeqbl( title )

* restore saved values
      if( changes ) then
      if( .false. ) then
	hjd0 = savehjd0
	t0 = hjd0 - mjd0
c	t0 = save( k0 )
	te = save( kt )
	smag0 = save( km )
	y0 = save( ku )
	sigmag0 = save( ks )
	erbscl = save( kf )
	b0 = save( kb )
	xpl = save( kx )
	ypl = save( ky )
	qpl = save( kq )
	npfit = nint( save( kn ) )
	ndata = nint( save( kd ) )

      if( .false. ) then
	last = nalways
      do i=1,nloc
	b0_l( i ) = save( last + ib0 )
	sigmag0_l( i ) = save( last + isigmag0 )
	erbscl_l( i ) = save( last + ierbscl )
* unsure if we need to save/restore addflx, addmag, sitefac ?
	if( iaddflx .gt. 0 ) addflx_l( i ) = save( last + iaddflx )
	if( iaddmag .gt. 0 ) addmag_l( i ) = save( last + iaddmag )
	if( isitefac .gt. 0 ) sitefac_l( i ) = save( last + isitefac )
	last = last + nperloc
      end do
      end if

      end if

* restore all original parameters
	call restoreall
* calculate chi2
	call chi2calc( bof, ndf )
	bof = bof / max( 1, ndf - npfit )
* report
	write(*,*) '----------'
     &	, ' FINAL CHI2/(', ndf, ' - ', npfit, ' ) = ', bof

c	call showpar
      end if
c	write(*,*) 'title_chi2 exit nalways', nalways, ' nperloc', nperloc

	return
	end
